/* revalidate.c */

#include<linux/string.h>
#include<linux/kernel.h>
#include"cache.h"
#include"structures.h"	
#include"hash.h"


extern int sys_time(void *);


int revalidate_lease(unsigned long inode_number, char *hostname, 
		     unsigned long new_lease, char *data, off_t offset, 
		     int count)
/*
  It's called when an old lease is validated for another period.
*/
{
  int size, last_byte;
  struct hash_table_entry *file;
  struct list_node *block;

  if (!(file  = find_hash_entry(inode_number, hostname)))
    {
      printk("The file was flushed out from the cache while the RPC was made.\n");
      return 0; /* The file was flushed out from the cache */
    }           /* while the RPC was made. No problem.     */

  if (file->lease_expiration < new_lease) /* Maybe another RPC has gotten a */ 
    file->lease_expiration = new_lease;   /* better lease.                  */
  
/* We'll do an awful thing now: we'll look for the cached block again. 
*/

  if(!(block = find_cache_block(file, offset)) ||
     block->offset + block->size <= offset)
    {
      printk("What a hell, the data aren't cached anymore :-(\n");
      return 0;   /* What a hell, the data aren't cached anymore :-( */
    }

  if (file->lease_expiration < sys_time(NULL))
    {
      /* What a hell, the lease came expired :-( */
      printk("What a hell, the lease came expired :-(\n");
      return 0;
    }

  last_byte = block->offset + block->size;
  if (offset + count < last_byte) last_byte = offset + count;
  size = last_byte - offset;
  memcpy(data, block->data + (offset - block->offset), size);
    
  refresh_block_LRU_list(block);

 return 1;                               
}



